package edu.whut.smilepicturebackend.manager;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.model.*;
import com.qcloud.cos.model.ciModel.persistence.PicOperations;
import edu.whut.smilepicturebackend.config.CosClientConfig;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.InputStream;


/**
 * 和业务没有关系，通用的文件上传下载
 */
@Component
public class CosManager {

    @Resource
    private CosClientConfig cosClientConfig;

    @Resource
    private COSClient cosClient;

    /**
     * 上传对象
     *
     * @param key  唯一键 ，相当于桶内的「文件路径」！
     * @param file 文件
     */
    public PutObjectResult putObject(String key, File file) {
        PutObjectRequest putObjectRequest = new PutObjectRequest(cosClientConfig.getBucket(), key,
                file);
        return cosClient.putObject(putObjectRequest);
    }

    /**
     * 上传对象（附带图片信息）
     *
     * @param key  唯一键
     * @param file 文件
     */
    public PutObjectResult putPictureObject(String key, File file) {
        PutObjectRequest putObjectRequest = new PutObjectRequest(cosClientConfig.getBucket(), key,
                file);
        // 对图片进行处理（获取基本信息也被视作为一种图片的处理）
        PicOperations picOperations = new PicOperations();
        // 1 表示返回原图信息
        picOperations.setIsPicInfo(1);
        // 构造处理参数
        putObjectRequest.setPicOperations(picOperations);
        return cosClient.putObject(putObjectRequest);
    }


    /**
     * 上传对象，接收InputStream文件流
     * @param key
     * @param in
     * @param contentLength
     * @param contentType
     */
    public PutObjectResult putObjectStream(String key,
                                           InputStream in,
                                           long contentLength,
                                           String contentType) {
        // 1. 构建 metadata
        ObjectMetadata meta = new ObjectMetadata();
        meta.setContentLength(contentLength);
        meta.setContentType(contentType);
        // （如果有额外的 metadata，比如缓存控制、Content-Disposition 等，也一并在这里设置）

        // 2. 构造 PutObjectRequest —— 支持 InputStream + metadata
        PutObjectRequest request = new PutObjectRequest(
                cosClientConfig.getBucket(),
                key,
                in,
                meta
        );

        // 3. 调用同一个接口上传
        return cosClient.putObject(request);
    }
    /**
     * 上传对象，接收InputStream文件流,附带图片信息
     * @param key
     * @param in
     * @param contentLength
     * @param contentType
     */
    public PutObjectResult putPictureObjectStream(
            String key,
            InputStream in,
            long contentLength,
            String contentType) {

        // 1. 构建基本的元数据
        ObjectMetadata meta = new ObjectMetadata();
        meta.setContentLength(contentLength);
        meta.setContentType(contentType);

        // 2. 构造带流的上传请求
        //    这里使用了流式的 PutObjectRequest 构造器
        PutObjectRequest request = new PutObjectRequest(
                cosClientConfig.getBucket(),
                key,
                in,
                meta
        );

        // 3. 设置图片处理操作：返回原图信息
        PicOperations picOps = new PicOperations();
        picOps.setIsPicInfo(1);
        request.setPicOperations(picOps);

        // 4. 执行上传
        return cosClient.putObject(request);
    }

    /**
     * 下载对象
     *
     * @param key 唯一键
     */
    public COSObject getObject(String key) {
        GetObjectRequest getObjectRequest = new GetObjectRequest(cosClientConfig.getBucket(), key);
        return cosClient.getObject(getObjectRequest);
    }


    /**
     * 删除对象
     *
     * @param key 唯一键
     */
    public void deleteObject(String key) {
        cosClient.deleteObject(cosClientConfig.getBucket(), key);
    }
}
